	subroutine vis

C	SUBROUTINE TO CALCULATE VISCOSITY OF THE MELT+CRYSTALS
C	AS A FUNCTION OF MELT COMPOSITION AND TEMPERATURE

C	DECLARE VARIABLES USED IN THIS SUBROUTINE

	implicit none

      real*8 beta,ceta,corrfact,ct,dh2o,eta,etagas,etagaslg,
     *		etamag,etamaglg,etamixlg,slope,tmoles,totalx,
     *		vex,vfgas,xtls
	real*8  Al2O3,CaO,Fe2O3,FeO,K2O,MgO,
     *		Na2O,SiO2,TiO2,mAl2O3,mCaO,mFe2O3,mFeO,mH2O,
     *		mK2O,mMgO,mNa2O,mSiO2,mTiO2,
     *		sAl2O3,sCaO,sFe2O3,sFeO,sH2O,sK2O,sMgO,
     *		sNa2O,sSiO2,sTiO2,
     *		xAl2O3,xCaO,xFe2O3,xFeO,xH2O,xK2O,
     *		xMgO,xNa2O,xSal,xSiO2,xTiO2

C	DECLARE VARIABLES LISTED IN THE COMMON BLOCK BUT NOT USED
C	IN THIS SUBROUTINE

      real*8 area(10000),mach(10000),pres(10000),
     *		re(10000),rho(10000),vel(10000),vesic(10000),
     *		visc(10000),wtoxan(9),z(10000)
      real*8  blkgas,blkmag,bubnum,Ca,cpm,cp,cph2o,
     *		cv,cvh2o,dadz,dfirst,dfinal,dhdp,diam,dpdz,dvdz,dz,
     *		emax,enthalpy,entropy,eps,
     *		exmol,f,fo,g,gamma,gh2o,h,hh2o,hf,m,mdot,mg,mm,
     *		msil,mw,mwttot,mxtls,
     *		pfinal,r,rey,rhof,rhom,rhomix,
     *		rhoxtls,s,sv,temp,v,vm,vxtls,vh2o,xsarea,zfinal
      integer i,icalc,iend,ifinal,ifrag,im,ip,io,
     *		iruns,itherm,ives

      Common/block1/area,mach,pres,re,
     *		rho,vel,vesic,visc,z
      Common/block2/beta,bubnum,Ca,cp,cv,
     *		dadz,dfinal,dfirst,dhdp,diam,dpdz,dvdz,dz,
     *		emax,eps,eta,etagas,etamag,
     *		exmol,f,fo,g,gamma,h,hf,m,mdot,mg,mh2o,mm,
     *		msil,mw,mxtls,pfinal,r,rey,rhof,rhomix,
     *		rhoxtls,s,
     *		sv,temp,v,vex,vfgas,vm,vxtls,xsarea,xtls,zfinal
      Common/block3/Al2O3,CaO,Fe2O3,FeO,K2O,MgO,
     *		Na2O,SiO2,TiO2
 	Common/ints/i,icalc,iend,ifinal,ifrag,im,ip,io,
     *		iruns,itherm,ives
 	common/melt/blkmag,cpm,dh2o,enthalpy,entropy,mwttot,rhom,wtoxan
	common/water/cpH2O,cvH2O,gH2O,hH2O,sH2O,vH2O,blkgas

	if (wtoxan(1).gt.70.) then

C	  IF THE WT% ANHYDROUS SIO2 IN THE MELT IS GREATER THAN 70%, 
C	  CALCULATE VISOCISTY USING RELATIONS FROM 
C	  HESS AND DINGWELL (1996):
		
		etamaglg = (-3.545+0.833*dlog(dh2o*100.))
     *			  + (9601.-2368.*dlog(dh2o*100.))
     *			 / (temp - (195.7 + 32.25*dlog(dh2o*100.)))
		etamag = 10.**etamaglg
	  else

C	  IF THE WT% SIO2 < 70%, USE RELATIONS FROM SHAW (1972).

C	  THE VARIABLES MSIO2, MAL2O3, ETC. ARE THE 
C	  MOLECULAR WEIGHTS OF EACH OXIDE (KG/MOLE), WITH THE 
C	  EXCEPTIONS THAT
C	  MAL2O3 AND MFE2O3 ARE PER GRAM-ATOM AL (I.E. HALF 
C	  THEIR MOLECULAR WEIGHTS), FOLLOWING THE METHODOLOGY 
C	  OF SHAW (1972).

		ceta = -6.40
		ct = 1.50
		mSiO2= .060084
		mAl2O3 = .050981
		mFe2O3 = .079845
		mFeO = .071846
		mMgO = .040304
		mCaO = .056077
		mTiO2 = .079878
		mNa2O = .061979
		mK2O = .094195
		mH2O = .018015

C	DECLARE VALUES OF EMPIRICAL COEFFICIENTS "Si" FROM SHAW (1972)
C	THESE CONSTANTS ARE USED TO DETERMINE THE SLOPE OF THE LN(VISCOSITY)
C	VERSUS TEMPERATURE CURVE FOR THE MAGMA.

		sAl2O3= 6.7
		sFe2O3 = 3.4
		sFeO = 3.4
		sMgO = 3.4
		sCaO = 4.5
		sTiO2 = 4.5
		sNa2O = 2.8
		sK2O = 2.8
		sH2O = 2.

C	CALCULATE TOTAL NUMBER OF MOLES OF OXIDES

		corrfact = 1.-dh2o
		tmoles = (SiO2/mSiO2 + Al2O3/mAl2O3 + Fe2O3/mFe2O3 
     *		+ FeO/mFeO + MgO/mMgO + CaO/mCaO + TiO2/mTiO2
     *		+ Na2O/mNa2O + K2O/mK2O) * corrfact
     *		+ 100.*dh2o/mH2O

C	CALCULATE MOLE FRACTIONS OF EACH OXIDE

		xSiO2 = corrfact*SiO2/(mSiO2*tmoles)
		xAl2O3 = corrfact*Al2O3/(mAl2O3*tmoles)
		xFe2O3 = corrfact*Fe2O3/(mFe2O3 *tmoles)
		xFeO = corrfact*FeO/(mFeO*tmoles)
		xMgO = corrfact*MgO/(mMgO*tmoles)
		xCaO = corrfact*CaO/(mCaO*tmoles)
		xTiO2 = corrfact*TiO2/(mTiO2*tmoles)
		xNa2O = corrfact*Na2O/(mNa2O*tmoles)
		xK2O = corrfact*K2O/(mK2O*tmoles)
		xH2O =  100.*dh2o/(mH2O*tmoles)

C		MAKE SURE MOLE FRACTIONS ALL ADD UP TO 1.

		totalx = xSiO2+xAl2O3+xFe2O3+xFeO+xMgO+xCaO+xTiO2+xNa2O+
     *		xK2O+xH2O

C		CALCULATE SLOPE OF ARRHENIAN CURVE FOR THE SHAW RELATION

		xsal = xSiO2/(1.-xSiO2)
		slope = sAl2O3*xAl2O3*xsal + sFe2O3*xFe2O3*xsal + 
     *	  sFeO*xFeO*xsal + sMgO*xMgO*xsal + sCaO*xCaO*xsal + 
     *	  sTiO2*xTiO2*xsal + sNa2O*xNa2O*xsal + sK2O*xK2O*xsal +
     *	  sH2O*xH2O*xsal

C		CALCULATE VISCOSITY 
C		SHAW'S CONSTANTS ARE VALID IF THE VISCOSITY IS IN POISE,
C		AND THE EQUATION IS FOR THE NATURAL LOG (NOT LOG(10)).

		etamaglg = slope*1.0e+04/temp - ct*slope + ceta
		etamag = exp(etamaglg)

C		CONVERT VISCOSITY FROM POISE TO PA S.

		etamag = etamag/10.
		etamaglg = dlog10(etamag)
	end if

C	 2.  CALCULATE VISCOSITY OF LIQUID PLUS CRYSTALS 
c	USING EINSTEIN-ROSCOE
C	EQUATION:	ETA = ETALIQ*(1.-PHI/0.6)**(-2.5)

	if (xtls.lt.59.) then
 	  etamag = etamag*(1.-xtls/60.)**(-2.5)
	end if

	return		
	end
